<?php

namespace Drupal\Tests\marketo_ma\Unit\Service;

use CSD\Marketo\Client as GuzzleClient;
use CSD\Marketo\Response;
use CSD\Marketo\Response\DeleteLeadResponse;
use Drupal\marketo_ma\Lead;
use Drupal\marketo_ma\Service\MarketoMaApiClient;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Psr\Log\LoggerInterface;

/**
 * @group marketo_ma
 * @coversDefaultClass \Drupal\marketo_ma\Service\MarketoMaApiClient
 */
class MarketoMaApiClientTest extends UnitTestCase {

  /**
   * @var \CSD\Marketo\Client|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $realClient;

  /**
   * The marketo_ma rest client service.
   *
   * @var \Drupal\marketo_ma\Service\MarketoMaApiClientInterface
   */
  protected $api_client;

  public function setUp() {
    parent::setUp();
    // Get the API client service.
    $this->realClient = $this->prophesize(GuzzleClient::class);
  }

  /**
   * @covers ::canConnect
   * @dataProvider provideConnect
   */
  public function testApiClientCanConnect($config, $expect) {
    $this->assertEquals($expect, $this->getClient($config)->canConnect());
  }

  /**
   * @covers ::getFields
   */
  public function testGetFields() {
    $field1 = ['rest' => ['name' => 'foo']];
    $field2 = ['rest' => ['name' => 'bar']];
    $soap_only = ['soap' => ['name' => 'bar']];
    $response = new Response([
      'result' => [
        $field1,
        $field2,
        $soap_only,
      ],
    ]);
    $this->realClient->describeLeads()
      ->shouldBeCalledOnce()
      ->willReturn($response);
    $fields = $this->getClient([])->getFields();
    // Assert fields match original fields plus default name. Why?
    $this->assertEquals([
      $field1,
      $field2,
      $soap_only,
    ], $fields);
  }

  /**
   * @covers ::getActivityTypes
   */
  public function testGetActivityTypes() {
    $expected_activities = [];
    $response = new Response([
      'result' => $expected_activities,
    ]);
    $this->markTestIncomplete('This does not exist on the class and so can not be mocked.');
    $this->realClient->getActivityTypes()
      ->shouldBeCalledOnce()
      ->willReturn($response);
    $activity_types = $this->getClient([])->getActivityTypes();
    // Assert fields match original fields plus default name. Why?
    $this->assertEquals($expected_activities, $activity_types);
  }

  /**
   * @covers ::getLeadById
   */
  public function testGetLeadById() {
    $lead = ['asdf'=>123];
    $response = new Response\GetLeadResponse([
      'success' => TRUE,
      'result' => [$lead],
    ]);
    $this->realClient->getLead(123)
      ->shouldBeCalledOnce()
      ->willReturn($response);
    $activity_types = $this->getClient([])->getLeadById(123);
    // Assert fields match original fields plus default name. Why?
    $this->assertEquals(new Lead($lead), $activity_types);
  }

  /**
   * @covers ::getLeadByEmail
   */
  public function testGetLeadByEmail() {
    $lead = ['asdf'=>123];
    $response = new Response\GetLeadResponse([
      'success' => TRUE,
      'result' => [$lead],
    ]);
    $this->realClient->getLeadByFilterType('email', 'test@example.com')
      ->shouldBeCalledOnce()
      ->willReturn($response);
    $activity_types = $this->getClient([])->getLeadByEmail('test@example.com');
    // Assert fields match original fields plus default name. Why?
    $this->assertEquals(new Lead($lead), $activity_types);
  }

  /**
   * @covers ::getLeadActivity
   */
  public function testGetLeadActivity() {
    $lead = new Lead(['id' => 123]);
    //xdebug_break();
    $this->realClient->getPagingToken(Argument::type('string'))
      ->shouldBeCalledOnce()
      ->willReturn(new Response\GetPagingToken([
      'success' => TRUE,
      'nextPageToken' => 'asdf1234',
    ]));
    $result = ['test' => 123];
    $response = new Response([
      'result' => $result
    ]);
    $this->realClient->getLeadActivity('asdf1234', 123, 1234)
      ->shouldBeCalledOnce()
      ->willReturn($response);
    $activity_types = $this->getClient([])->getLeadActivity($lead, 1234);
    // Assert fields match original fields plus default name. Why?
    $this->assertEquals($result, $activity_types);
  }

  /**
   * @covers ::syncLead
   */
  public function testSyncLead() {
    $lead_data = ['id' => 543, 'name' => 'test'];
    $lead = new Lead($lead_data);
    $this->realClient->createOrUpdateLeads([$lead_data], 'email', [])
      ->shouldBeCalledOnce()
      ->willReturn(new Response\CreateOrUpdateLeadsResponse(['result' => $lead_data]));
    $this->assertEquals($lead_data, $this->getClient([])->syncLead($lead));
  }

  /**
   * @covers ::deleteLead
   */
  public function testDeleteLead() {
    $leads = [123, 234, 345];
    $this->realClient->deleteLead($leads)
      ->shouldBeCalledOnce()
      ->willReturn(new DeleteLeadResponse(['result' => TRUE]));
    $this->assertTrue($this->getClient([])->deleteLead($leads));
  }

  /**
   * @covers ::addLeadsToList
   * @covers ::getLeadsIds
   */
  public function testAddLeadsToList() {
    $leads_raw = [123, 234, 345];
    $leads = array_map(function ($id) { return new Lead(['id' => $id]); }, $leads_raw);

    $options = ['myoption' => 'my value'];
    $this->realClient->addLeadsToList(789, implode(',', $leads_raw), $options)
      ->shouldBeCalledOnce()
      ->willReturn(new Response\AddOrRemoveLeadsToListResponse(['result' => TRUE]));
    $this->assertTrue($this->getClient([])->addLeadsToList(789, $leads, $options));

    $this->realClient->addLeadsToList(789, implode(',', $leads_raw), [])
      ->shouldBeCalledOnce()
      ->willReturn(new Response\AddOrRemoveLeadsToListResponse(['result' => TRUE]));
    $this->assertTrue($this->getClient([])->addLeadsToList(789, $leads));
    // TODO add a test for exception handling in getLeadsIds
  }

  /**
   * @covers ::addLeadToListByEmail
   */
  public function testAddLeadToListByEmail() {
    $lead_data = ['id' => 543, 'name' => 'test'];
    $lead_response = new Response\GetLeadResponse([
      'success' => TRUE,
      'result' => [$lead_data],
    ]);
    $this->realClient->getLeadByFilterType('email', 'test@example.com')
      ->willReturn($lead_response);

    $this->realClient->createOrUpdateLeads(Argument::any(), Argument::any(), Argument::any())
      ->willReturn(new Response\CreateOrUpdateLeadsResponse(['result' => []]));

    $options = ['myoption' => 'my value'];
    $this->realClient->addLeadsToList(789, '543', $options)
      ->shouldBeCalledOnce()
      ->willReturn(new Response\AddOrRemoveLeadsToListResponse(['result' => TRUE]));
    $this->getClient([])->addLeadToListByEmail(789, 'test@example.com', $options);

    $this->realClient->addLeadsToList(789, '543', [])
      ->shouldBeCalledOnce()
      ->willReturn(new Response\AddOrRemoveLeadsToListResponse(['result' => TRUE]));
    $this->getClient([])->addLeadToListByEmail(789, 'test@example.com');
  }


  private function getClient($config) {
    return new class (
      $config,
      $this->prophesize(LoggerInterface::class)->reveal(),
      $this->realClient->reveal()) extends
      MarketoMaApiClient {

      public function __construct(
        $config,
        LoggerInterface $logger,
        GuzzleClient $apiClient
      ) {
        $this->client_config = $config;
        $this->logger = $logger;
        $this->client = $apiClient;
      }

    };
  }

  public function provideConnect() {
    return [
      [
        [
          'munchkin_id' => 'a',
          'client_id' => 'a',
        ],
        TRUE,
      ],
      [
        [],
        FALSE,
      ],
    ];
  }

}
